<?php
namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities\DateUtil;
use Teknisa\Libs\Util\Environment; 
use Teknisa\MAN\Factories\Ger96500MonitoramentoFechamentoCupomFactory;
use Teknisa\MAN\Services\Ger97101FechamentoCupomService;


class Ger96500MonitoramentoFechamentoCupomService {

    protected $environment;
    protected $fechamentoService;
    protected $monitoramentoFactory;

    public function __construct(Environment $environment, Ger97101FechamentoCupomService $fechamentoService, Ger96500MonitoramentoFechamentoCupomFactory $monitoramentoFactory) {
        $this->environment = $environment;
        $this->fechamentoService = $fechamentoService;
        $this->monitoramentoFactory = $monitoramentoFactory;
    }

    /*
     * Função auxiliar para contar a quantidade de lançamentos aprovados
     * e pendentes retornados.
     */
    private function countAprovacoes($data, &$counter, $countFechadas) {
        foreach ($data as $cupom) {
            // Verifica se o cupom está aprovado ou fechado sem movimentação.
            if ($cupom["IDSTATUS"] == 'A' || ($cupom["IDSTATUS"] == 'F' && $countFechadas)) {
                $counter["CONCLUIDAS"]++;
            }
            // Verifica se o cupom está pendente ou com caixa sem manutenção.
            else if ($cupom["IDSTATUS"] == 'P' || $cupom["IDSTATUS"] == 'M') {
                $counter["PENDENTES"]++;
            }
        }
    }

    /*
     * Função para buscar os cupons por dia das filiais parametrizadas.
     * Executa a função de busca uma vez para cada dia no intervalo dado.
     */
    public function getCuponsDistinctFiliais($filter) {
        $cupons = array();
        $params = array(
            "CDFILIAL" => $filter["CDFILIAL"],
            "NRORG" => $this->environment->getNrorg()
        );
        // Verifica se a data inicial e final foram inseridas.
        // Entra nesse if quando se quer filtrar por um intervalo de datas,
        // buscando somente as filiais distintas.
        if (!(empty($filter["DATAINI"]) || empty($filter["DATAFIN"]))) {
            // Inicializa variáveis de contabilização
            $aprovacoes = array(
                "CONCLUIDAS" => 0,
                "PERCENTCONCLUIDAS" => 0,
                "PENDENTES" => 0,
                "PERCENTPENDENTES" => 0,
            );
            $filiais = array(
                "CONCLUIDAS" => 0,
                "PERCENTCONCLUIDAS" => 0,
                "PENDENTES" => 0,
                "PERCENTPENDENTES" => 0,
            );
            $dataInicial = DateUtil::getDataDeString($filter["DATAINI"]);
            $dataFinal = DateUtil::getDataDeString($filter["DATAFIN"]);
            while (!DateUtil::comparaDatas($dataInicial, $dataFinal, DateUtil::DEPOIS)) {
                // Utiliza data atual do loop para filtrar.
                $dtreducaoz = $dataInicial->format(DateUtil::FORMATO_BRASILEIRO);
                $params["DTREDUCAOZ"] = $dtreducaoz;
                $data = $this->monitoramentoFactory->getCuponsPrincipal($params);
                // Conta total de cupons por caixa.
                $data = $this->filterDistinctFiliais($data, true);
                $this->countAprovacoes($data, $aprovacoes, false);

                // Filtra os cupons por filial.
                $data = $this->filterDistinctFiliais($data, false);

                // Adiciona os cupons filtrados ao array de cupons retornados.
                $cupons = array_merge($cupons, $data);

                // Adiciona um dia ao loop.
                $dataInicial = DateUtil::adicionaIntervalo($dataInicial, 1, DateUtil::DIA);
            }
            // Conta total de cupons por filial.
            $data = $this->filterDistinctFiliais($cupons, false);
            $this->countAprovacoes($data, $filiais, true);

            // Verifica o total de cupons e de filiais.
            $aprovacoes["TOTAL"] = $aprovacoes["CONCLUIDAS"] + $aprovacoes["PENDENTES"];
            $filiais["TOTAL"] = $filiais["CONCLUIDAS"] + $filiais["PENDENTES"];
            // Se houver um número de cupons maior que zero, realiza a
            // contabilização.
            if ($aprovacoes["TOTAL"]) {
                $aprovacoes["PERCENTCONCLUIDAS"] = round(($aprovacoes["CONCLUIDAS"] / $aprovacoes["TOTAL"]) * 100, 2);
                $aprovacoes["PERCENTPENDENTES"] = round(($aprovacoes["PENDENTES"] / $aprovacoes["TOTAL"]) * 100, 2);
                $filiais["PERCENTCONCLUIDAS"] = round(($filiais["CONCLUIDAS"] / $filiais["TOTAL"]) * 100, 2);
                $filiais["PERCENTPENDENTES"] = round(($filiais["PENDENTES"] / $filiais["TOTAL"]) * 100, 2);
            }
            // Insere a contabilização no início do array de cupons.
            array_unshift($cupons, array("aprovacoes" => $aprovacoes, "filiais" => $filiais));
        }
        // Retorna os dados obtidos.
        return $cupons;
    }

    /*
     * Função para buscar os cupons de uma filial, exibindo todos os cupons
     * de um determinado dia para essa filial.
     */
    public function getCuponsDistinctCaixas($filter) {
        $params = array(
            "DTREDUCAOZ" => $filter["DTREDUCAOZ"],
            "CDFILIAL" => $filter["CDFILIAL"],
            "NRORG" => $this->environment->getNrorg()
        );
        $cupons = $this->monitoramentoFactory->getCuponsPorCaixa($params);
        // Filtra os cupons por filial e caixa.
        $cupons = $this->filterDistinctFiliais($cupons, true);

        return $cupons;
    }

    /*
     * Função para retornar os erros de pré-validação de um cupom pendente.
     * Executa a procedure de pré-validação caso não haja dados previamente
     * armazenados no banco de dados.
     */
    public function getErrosPreValidacao($cupom) {
        $errosPreValidacao = array();

        // Verifica se o cupom está pendente.
        if ($cupom["IDSTATUS"] == "P") {
            $params = array(
                "DTREDUCAOZ" => $cupom["DTREDUCAOZ"],
                "CDFILIAL" => $cupom["CDFILIAL"],
                "CDCAIXA" => $cupom["CDCAIXA"],
                "NRORG" => $cupom["NRORG"]
            );
            // Busca dados no banco de dados.
            $errosPreValidacao = $this->monitoramentoFactory->getErrosPreValidacao($params);

            // Se nenhum dado for encontrado, realiza rotina de pré-validação.
            if (empty($errosPreValidacao)) {
                $errosPreValidacao = $this->fechamentoService->preValidarCuponsFiscais(array($cupom));
            }
        }

        return $errosPreValidacao;

    }

    /*
     * Função auxiliar que filtra os cupons de forma distinta. Ela é utilizada
     * para que uma determinada filial seja identificada num dia de acordo com
     * o status de maior precedência na ordem M > P > A. Como somente um cupom
     * pode ser retornado com o status F (quando não há cupons para o dia),
     * então não é necessário verificar esse status. Pode ser utilizado para
     * filtrar por filial e caixa, também.
     * Exemplo 1: Uma filial tem, num dia, três cupons, sendo dois aprovados e
     * um pendente. A filial será identificada, nesse dia, como pendente, pois
     * há um cupom pendente.
     * Exemplo 2: Uma filial tem, num dia, três cupons, sendo um aprovado, um
     * pendente e um sem o fechamento de caixa. A filial será identificada,
     * nesse dia, como sem fechamento de caixa, pois há um cupom nesse estado.
     */
    private function filterDistinctFiliais($data, $distinctCaixa = false) {
        // Variável auxiliar que irá armazenar os cupons filtrados.
        $aux = array();
        // Array de índices dos cupons que devem ser utilizados após filtrar.
        $indicesToKeep = array();
        foreach ($data as $key => $cupom) {
            // Verifica qual é o índice único do cupom, de acordo com a flag
            // $distinctCaixa, que permite que o índice leve o caixa em
            // consideração.
            $distinctIndex = $distinctCaixa ? $cupom["CDFILIAL"] . $cupom["CDCAIXA"] : $cupom["CDFILIAL"];
            if (empty($indicesToKeep[$distinctIndex])) {
                // Se ainda não houver cupom verificado para esse índice, adiciona-o.
                $indicesToKeep[$distinctIndex] = $key;
            }
            // Se já houver cupom verificado para esse índice, mas o cupom
            // atual for de status 'M' (maior prioridade), adiciona-o.
            else if ($cupom["IDSTATUS"] == 'M') {
                $indicesToKeep[$distinctIndex] = $key;
            }
            // Se já houver cupom verificado para esse índice, mas o cupom
            // atual for de status 'P' (segunda maior prioridade), e o cupom
            // já armazenado não possuir prioridade maior, adiciona-o.
            else if ($cupom["IDSTATUS"] == 'P' && $data[$indicesToKeep[$distinctIndex]]["IDSTATUS"] != 'M') {
                $indicesToKeep[$distinctIndex] = $key;
            }
            // Caso não entre em nenhum dos if's, o cupom será ignorado.
        }
        // Armazena todos os cupons filtrados na variável auxiliar.
        foreach ($indicesToKeep as $index) {
            $aux[] = $data[$index];
        }
        return $aux;
    }
}